@@ -46,6 +46,11 @@
 //! let bitvector4 = bitvector & bitvector2;
 //! // ofcourse you can just use bit-and operator on bitvectors, it will also consumes the inputs.
 //! assert_eq!(bitvector3, bitvector4);
+//! // A bitvector can also be constructed from a collection of bool, or a colluction of integer:
+//! let bitvector: BitVector = (0 .. 10).map(|x| x%2 == 0).into();
+//! let bitvector2: BitVector = (0 .. 10).map(|x| x%3 == 0).into();
+//! let bitvector3 = BitVector::from_bool_iterator((0..10).map(|x| x%6 == 0));
+//! assert_eq!(bitvector & bitvector2, bitvector3)
 //! ```
 //!
 //! ## Performance
@@ -99,7 +104,7 @@ macro_rules! impl_operation {
             .map(|(a, b)| a $op b)
             .collect();
         Self {
-            storage: storage,
+            storage,
             nbits: self.nbits,
         }
         }
@@ -113,7 +118,7 @@ macro_rules! impl_operation {
             .map(|(a, b)| a $op b)
             .collect();
         Self {
-            storage: storage,
+            storage,
             nbits: self.nbits,
         }
         }
@@ -177,10 +182,7 @@ impl BitVector {
         let (len, bytes, bits) = bit_to_len(nbits);
         let len = if bytes > 0 || bits > 0 { len + 1 } else { len };
         let storage = (0..len).map(|_| BitContainer::splat(0)).collect();
-        Self {
-            storage: storage,
-            nbits: nbits,
-        }
+        Self { storage, nbits }
     }
 
     /// Create a bitvector containing all 0 .. nbits elements.
@@ -198,10 +200,7 @@ impl BitVector {
             assert_eq!(slice.len(), 8);
             storage.push(BitContainer::from_slice_unaligned(&slice));
         }
-        Self {
-            storage: storage,
-            nbits: nbits,
-        }
+        Self { storage, nbits }
     }
 
     /// Create a bitvector from an Iterator of bool.
@@ -211,25 +210,25 @@ impl BitVector {
     /// ```rust
     /// use bitvector_simd::BitVector;
     ///
-    /// let bitvector = BitVector::from_iterator((0..10).map(|x| x % 2 == 0));
+    /// let bitvector = BitVector::from_bool_iterator((0..10).map(|x| x % 2 == 0));
     /// assert_eq!(bitvector.capacity(), 10);
     /// assert_eq!(<BitVector as Into<Vec<bool>>>::into(bitvector), vec![true, false, true, false, true, false, true, false, true, false]);
     ///
-    /// let bitvector = BitVector::from_iterator((0..1000).map(|x| x < 50));
+    /// let bitvector = BitVector::from_bool_iterator((0..1000).map(|x| x < 50));
     /// assert_eq!(bitvector.capacity(), 1000);
     /// assert_eq!(bitvector.get(49), Some(true));
     /// assert_eq!(bitvector.get(50), Some(false));
     /// assert_eq!(bitvector.get(999), Some(false));
     /// assert_eq!(<BitVector as Into<Vec<bool>>>::into(bitvector), (0..1000).map(|x| x<50).collect::<Vec<bool>>());
     /// ```
-    pub fn from_iterator<I: Iterator<Item = bool>>(mut i: I) -> Self {
+    pub fn from_bool_iterator<I: Iterator<Item = bool>>(i: I) -> Self {
         // FIXME: any better implementation?
         let mut storage = Vec::new();
         let mut current_slice = [0u64; 8];
         let mut nbits = 0;
-        while let Some(b) = i.next() {
+        for b in i {
             if b {
-                current_slice[nbits / 64] |= 1 << (u64::BITS - (nbits%64) as u32 - 1);
+                current_slice[nbits / 64] |= 1 << (u64::BITS - (nbits % 64) as u32 - 1);
             }
             nbits += 1;
             if nbits % 512 == 0 {
@@ -243,6 +242,24 @@ impl BitVector {
         Self { storage, nbits }
     }
 
+    /// Initialize from a set of integers.
+    ///
+    /// Example:
+    ///
+    /// ```rust
+    /// use bitvector_simd::BitVector;
+    ///
+    /// let bitvector = BitVector::from_slice(&[0,5,9]);
+    /// assert_eq!(<BitVector as Into<Vec<bool>>>::into(bitvector), vec![true, false, false, false, false, true, false, false, false, true]);
+    /// ```
+    pub fn from_slice(slice: &[usize]) -> Self {
+        let mut bv = BitVector::zeros(slice.len());
+        for i in slice {
+            bv.set(*i, true);
+        }
+        bv
+    }
+
     /// Max number of elements that this bitvector can have.
     ///
     /// To get the number of elements, use `count`
@@ -332,6 +349,11 @@ impl BitVector {
     /// ```rust
     /// use bitvector_simd::BitVector;
     ///
+    /// let bitvector : BitVector = (0 .. 15).map(|x| x%3 == 0).into();
+    /// assert_eq!(bitvector.get(3), Some(true));
+    /// assert_eq!(bitvector.get(5), Some(false));
+    /// assert_eq!(bitvector.get(14), Some(false));
+    /// assert_eq!(bitvector.get(15), None);
     /// ```
     pub fn get(&self, index: usize) -> Option<bool> {
         if self.nbits <= index {
@@ -344,10 +366,21 @@ impl BitVector {
 
     /// Directly return a `bool` instead of an `Option`
     ///
-    /// * If exists, return `true`
-    /// * If doesn't exist or index >= current.capacity, retun `false`
+    /// * If exists, return `true`.
+    /// * If doesn't exist, return false.
+    /// * If index >= current.capacity, panic.
     ///
-    /// equals to `self.get(index).unwrap_or(false)`
+    ///
+    /// Examlpe:
+    ///
+    /// ```rust
+    /// use bitvector_simd::BitVector;
+    ///
+    /// let bitvector : BitVector = (0 .. 15).map(|x| x%3 == 0).into();
+    /// assert_eq!(bitvector.get_unchecked(3), true);
+    /// assert_eq!(bitvector.get_unchecked(5), false);
+    /// assert_eq!(bitvector.get_unchecked(14), false);
+    /// ```
     pub fn get_unchecked(&self, index: usize) -> bool {
         if self.nbits <= index {
             panic!("index out of bounds {} > {}", index, self.nbits);
@@ -389,7 +422,7 @@ impl BitVector {
     ///
     /// If your bitvector have capacity `1_000` and contains `[1,5]`,
     /// after inverse it will contains `0, 2..=4, 6..=999`
-    pub fn not(self) -> Self {
+    pub fn inverse(self) -> Self {
         let (i, bytes, bits) = bit_to_len(self.nbits);
         let mut storage = self.storage.into_iter().map(|x| !x).collect::<Vec<_>>();
         if bytes > 0 || bits > 0 {
@@ -408,7 +441,7 @@ impl BitVector {
         }
 
         Self {
-            storage: storage,
+            storage,
             nbits: self.nbits,
         }
     }
@@ -447,11 +480,11 @@ impl BitVector {
     /// Consume self and generate a `Vec<bool>` with length == self.capacity().
     ///
     /// Example:
-    /// 
+    ///
     /// ```rust
     /// use bitvector_simd::BitVector;
     ///
-    /// let bitvector = BitVector::from_iterator((0..10).map(|i| i % 3 == 0));
+    /// let bitvector = BitVector::from_bool_iterator((0..10).map(|i| i % 3 == 0));
     /// let bool_vec = bitvector.into_bools();
     /// assert_eq!(bool_vec, vec![true, false, false, true, false, false, true, false, false, true])
     /// ```
@@ -465,8 +498,8 @@ impl BitVector {
     ///
     /// ```rust
     /// use bitvector_simd::BitVector;
-    /// 
-    /// let bitvector = BitVector::from_iterator((0..10).map(|i| i%3 == 0));
+    ///
+    /// let bitvector = BitVector::from_bool_iterator((0..10).map(|i| i%3 == 0));
     /// let usize_vec = bitvector.into_usizes();
     /// assert_eq!(usize_vec, vec![0,3,6,9]);
     /// ```
@@ -477,6 +510,20 @@ impl BitVector {
     /// Only compare the first `bits` instead of the whole bitvector.
     ///
     /// Require self and other are both no shorter than `bits`.
+    ///
+    /// Example:
+    /// 
+    /// ```rust
+    /// use bitvector_simd::BitVector;
+    /// 
+    /// let bitvector = BitVector::from_slice(&[1,3,5,7]);
+    /// let bitvector2 = BitVector::from_slice(&[1,3,5,9,10,15]);
+    /// // compare first 6 bits (0..=5)
+    /// assert!(bitvector.eq_left(&bitvector2, 6));
+    /// // compare first 8 bits (0..=7)
+    /// assert!(!bitvector.eq_left(&bitvector2, 8));
+    /// // any bits > 8 call panic.
+    /// ```
     pub fn eq_left(&self, other: &Self, bits: usize) -> bool {
         assert!(self.nbits >= bits && other.nbits >= bits);
         let (i, bytes, bits) = bit_to_len(bits);
@@ -500,15 +547,15 @@ impl BitVector {
     }
 }
 
-impl<I:Iterator<Item=bool>> From<I> for BitVector {
+impl<I: Iterator<Item = bool>> From<I> for BitVector {
     fn from(i: I) -> Self {
-        Self::from_iterator(i)
+        Self::from_bool_iterator(i)
     }
 }
 
-impl Into<Vec<bool>> for BitVector {
-    fn into(self) -> Vec<bool> {
-        self.storage
+impl From<BitVector> for Vec<bool> {
+    fn from(v: BitVector) -> Self {
+        v.storage
             .into_iter()
             .flat_map(|x| {
                 let mut slice = [0u64; 8];
@@ -518,14 +565,14 @@ impl Into<Vec<bool>> for BitVector {
                 slice
             })
             .flat_map(|x| (0..u64::BITS).map(move |i| (x >> (u64::BITS - i - 1)) & 1 > 0))
-            .take(self.nbits)
+            .take(v.nbits)
             .collect()
     }
 }
 
-impl Into<Vec<usize>> for BitVector {
-    fn into(self) -> Vec<usize> {
-        self.storage
+impl From<BitVector> for Vec<usize> {
+    fn from(v: BitVector) -> Self {
+        v.storage
             .into_iter()
             .flat_map(|x| {
                 let mut slice = [0u64; 8];
@@ -535,7 +582,7 @@ impl Into<Vec<usize>> for BitVector {
                 slice
             })
             .flat_map(|x| (0..u64::BITS).map(move |i| (x >> (u64::BITS - i - 1)) & 1 > 0))
-            .take(self.nbits)
+            .take(v.nbits)
             .enumerate()
             .filter(|(_, b)| *b)
             .map(|(i, _)| i)
@@ -610,7 +657,7 @@ impl BitXor for BitVector {
 impl Not for BitVector {
     type Output = Self;
     fn not(self) -> Self::Output {
-        self.not()
+        self.inverse()
     }
 }
 
